@@ -28,13 +28,11 @@
 #include <sharedkvtmlfiles.h>
 #include <keduvocdocument.h>
 #include <keduvocexpression.h>
-#ifdef BUILD_WITH_SPEECH
-#include <kspeech.h>
-#include <ktoolinvocation.h>
+#ifdef HAVE_SPEECH
+#include <QTextToSpeech>
 #endif
 
 #include <KLocalizedString>
-#include <phonon/MediaObject>
 #include <sonnet/speller.h>
 
 #include <QLocale>
@@ -44,11 +42,12 @@
 KanagramGame::KanagramGame()
     : m_fileIndex(-1)
       ,m_document(NULL)
-      ,m_player(NULL)
-#ifdef BUILD_WITH_SPEECH
-      ,m_kspeech(NULL)
+#ifdef HAVE_SPEECH
+      ,m_speech(NULL)
 #endif
       ,m_totalScore(0)
+      ,m_totalScore2(0)
+      ,m_currentPlayerNumber(1)
       ,m_speller(NULL)
 {
     loadSettings();
@@ -58,8 +57,8 @@ KanagramGame::KanagramGame()
 
     // Load the default vocabulary
     loadDefaultVocabulary();
-#ifdef BUILD_WITH_SPEECH
-    setupJovie();
+#ifdef HAVE_SPEECH
+    m_speech = new QTextToSpeech(this);
 #endif
 
     m_speller = new Sonnet::Speller();
@@ -73,10 +72,12 @@ KanagramGame::~KanagramGame()
 
     delete m_document;
     m_document = NULL;
-    delete m_player;
-    m_player = NULL;
     delete m_speller;
     m_speller = NULL;
+#ifdef HAVE_SPEECH
+    delete m_speech;
+    m_speech = NULL;
+#endif
 }
 
 bool KanagramGame::checkFile()
@@ -93,9 +94,9 @@ bool KanagramGame::checkFile()
 QString KanagramGame::sanitizedDataLanguage() const
 {
     QString dataLanguage = KanagramSettings::dataLanguage();
+    QStringList languageCodes = SharedKvtmlFiles::languages();
 
-    if (dataLanguage.isEmpty()) {
-        QStringList languageCodes = SharedKvtmlFiles::languages();
+    if (dataLanguage.isEmpty() || !languageCodes.contains(dataLanguage)) {
         if (languageCodes.contains(QLocale::system().uiLanguages().at(0))) {
             dataLanguage = QLocale::system().uiLanguages().at(0);
         } else {
@@ -117,6 +118,28 @@ void KanagramGame::loadDefaultVocabulary()
     nextAnagram();
 }
 
+void KanagramGame::setSinglePlayerMode(bool singlePlayer)
+{
+    KanagramSettings::setSinglePlayerMode(singlePlayer);
+    emit singlePlayerChanged();
+}
+
+bool KanagramGame::singlePlayerMode()
+{
+  return KanagramSettings::singlePlayerMode();
+}
+
+int KanagramGame::getPlayerNumber()
+{
+    return m_currentPlayerNumber;
+}
+
+void KanagramGame::setPlayerNumber(int pnumber)
+{
+    m_currentPlayerNumber = pnumber;
+    emit currentPlayerChanged();
+}
+
 bool KanagramGame::refreshVocabularyList()
 {
     QString oldFilename = m_filename;
@@ -230,6 +253,7 @@ void KanagramGame::nextAnagram()
             m_audioUrl = "";
             // TODO: add some error reporting here
         }
+        emit userAnswerChanged();
         emit wordChanged();
     }
 }
@@ -239,9 +263,15 @@ QString KanagramGame::filename() const
     return m_fileList.isEmpty() ? m_filename : m_fileList.at(m_fileIndex);
 }
 
-QString KanagramGame::anagram() const
+QStringList KanagramGame::anagram() const
 {
-    return m_anagram;
+    QStringList resultList;
+    foreach (const QChar &userLetter, m_anagram)
+    {
+        resultList.append(userLetter);
+    }
+
+    return resultList;
 }
 
 QString KanagramGame::hint() const
@@ -254,6 +284,17 @@ QString KanagramGame::word() const
     return m_originalWord;
 }
 
+QStringList KanagramGame::userAnswer() const
+{
+    QStringList returnList;
+    foreach (const QChar &letter, m_userAnswer)
+    {
+        returnList.append(letter);
+    }
+    return returnList;
+}
+
+
 void KanagramGame::createAnagram()
 {
     if (m_originalWord.count(m_originalWord.at(0)) < m_originalWord.size()) {
@@ -273,14 +314,16 @@ void KanagramGame::createAnagram()
         } while (anagram == m_originalWord);
 
         m_anagram = anagram;
+        m_userAnswer.clear();
     } else {
         m_anagram = m_originalWord;
+        m_userAnswer.clear();
     }
+}
 
-    if (KanagramSettings::useSounds())
-    {
-        play("chalk.ogg");
-    }
+bool KanagramGame::useSounds()
+{
+    return KanagramSettings::useSounds();
 }
 
 QString KanagramGame::documentTitle() const
@@ -347,69 +390,12 @@ QUrl KanagramGame::audioFile()
     return m_audioUrl;
 }
 
-void KanagramGame::play(const QString& filename)
-{
-    if (!filename.isEmpty())
-    {
-        QString soundFile = QStandardPaths::locate(QStandardPaths::DataLocation, "sounds/" + filename);
-        if (soundFile.isEmpty())
-            soundFile = filename;
-
-        if (!m_player)
-        {
-            m_player = Phonon::createPlayer(Phonon::GameCategory, QUrl::fromLocalFile(soundFile));
-        }
-        else
-        {
-            m_player->setCurrentSource(QUrl::fromLocalFile(soundFile));
-        }
-        m_player->play();
-    }
-}
-
+#ifdef HAVE_SPEECH
 void KanagramGame::wordRevealed()
 {
     if (KanagramSettings::enablePronunciation())
     {
-        // User wants words spoken, but if there's no audio file, play right.ogg
-        if (!m_audioUrl.isEmpty())
-            play(m_audioUrl.toLocalFile());
-#ifdef BUILD_WITH_SPEECH
-        else
-            say(m_originalWord);
-#else
-        else
-            play("right.ogg");
-#endif
-    }
-}
-
-#ifdef BUILD_WITH_SPEECH
-void KanagramGame::setupJovie()
-{
-    // If KTTSD not running, start it.
-    QDBusReply<bool> reply = QDBusConnection::sessionBus().interface()->isServiceRegistered( "org.kde.kttsd" );
-    bool kttsdactive = false;
-    if ( reply.isValid() )
-        kttsdactive = reply.value();
-    if ( !kttsdactive )
-    {
-        QString error;
-        if ( KToolInvocation::startServiceByDesktopName( "kttsd", QStringList(), &error ) )
-        {
-            QMessageBox::warning(NULL, i18n("Speech System Failure"), i18n( "Starting Jovie Text-to-Speech service Failed: %1", error ) );
-        }
-        else
-        {
-            kttsdactive = true;
-        }
-    }
-    if ( kttsdactive && m_kspeech==0)
-    {
-        // creating the connection to the kspeech interface
-        m_kspeech = new org::kde::KSpeech( "org.kde.kttsd", "/KSpeech", QDBusConnection::sessionBus() );
-        m_kspeech->setParent(this);
-        m_kspeech->setApplicationName( "Kanagram" );
+        say(m_originalWord);
     }
 }
 
@@ -418,10 +404,9 @@ void KanagramGame::say(QString text)
     if ( text.isEmpty() )
         return;
 
-    setupJovie();
-    if ( this->m_kspeech )
+    if ( m_speech )
     {
-        QDBusReply< int > reply = this->m_kspeech->say(text, KSpeech::soPlainText );
+        m_speech->say(text);
     }
 }
 #endif
@@ -450,6 +435,46 @@ int KanagramGame::scoreTime()
     return ((scoreTime + 1) * 15);
 }
 
+void KanagramGame::moveLetterToUserAnswer(int position)
+{
+    m_userAnswer.append(m_anagram[position]);
+    m_anagram.remove(position, 1);
+    emit wordChanged();
+    emit userAnswerChanged();
+}
+
+void KanagramGame::moveLetterToAnagram(int position)
+{
+    m_anagram.append(m_userAnswer[position]);
+    m_userAnswer.remove(position, 1);
+    emit wordChanged();
+    emit userAnswerChanged();
+}
+
+void KanagramGame::resetAnagram()
+{
+    m_anagram = m_userAnswer;
+    m_userAnswer.clear();
+    emit wordChanged();
+    emit userAnswerChanged();
+}
+
+void KanagramGame::moveLetter(QString letter)
+{
+    QString small = letter.toLower();
+    int index = m_anagram.toLower().indexOf(small);
+    if (index != -1)
+    {
+        moveLetterToUserAnswer(index);
+    }
+    else
+    {
+        index = m_userAnswer.toLower().indexOf(letter);
+        if (index != -1)
+            moveLetterToAnagram(index);
+    }
+}
+
 int KanagramGame::getNumericSetting(QString settingString)
 {
     int indexFound_setting = settingString.size();
@@ -466,13 +491,20 @@ int KanagramGame::getNumericSetting(QString settingString)
 
 void KanagramGame::resetTotalScore()
 {
-    m_totalScore = 0;
-    emit scoreChanged();
+    if (m_currentPlayerNumber == 1)
+    {
+        m_totalScore = 0;
+        m_totalScore2 = 0;
+        emit scoreChanged();
+    }
 }
 
 void KanagramGame::adjustScore(int points)
 {
-    m_totalScore += points;
+    if (m_currentPlayerNumber == 1)
+        m_totalScore += points;
+    else
+        m_totalScore2 += points;
     emit scoreChanged();
 }
 
@@ -481,15 +513,20 @@ int KanagramGame::totalScore()
     return m_totalScore;
 }
 
+int KanagramGame::totalScore2()
+{
+    return m_totalScore2;
+}
+
 void KanagramGame::revealWord()
 {
     m_anagram = m_originalWord;
     emit wordChanged();
 }
 
-bool KanagramGame::checkWord(QString answer)
+bool KanagramGame::checkWord()
 {
-    QString enteredWord = answer.toLower().trimmed();
+    QString enteredWord = m_userAnswer.toLower().trimmed();
     QString lowerOriginal = m_originalWord.toLower();
     QString strippedOriginal = stripAccents(m_originalWord);
     if (!enteredWord.isEmpty())
@@ -500,27 +537,24 @@ bool KanagramGame::checkWord(QString answer)
                (isAnagram(enteredWord, lowerOriginal) ||
                 isAnagram(enteredWord, strippedOriginal))))
         {
+#ifdef HAVE_SPEECH
             if (KanagramSettings::enablePronunciation())
             {
-                // User wants words spoken, but if there's no audio file, play right.ogg
-                if (!m_audioUrl.isEmpty())
-                    play(m_audioUrl.toLocalFile());
-            }
-            else if (KanagramSettings::useSounds())
-            {
-                // Otherwise just play right.ogg
-                play("right.ogg");
+                // User wants words spoken
+                say(m_originalWord);
             }
+#endif
             return true;
         }
         else
         {
-            if (KanagramSettings::useSounds()) play("wrong.ogg");
             return false;
         }
     }
     else
+    {
         return false;
+    }
 }
 
 bool KanagramGame::isAnagram(QString& enteredword, QString& word)
@@ -613,24 +647,36 @@ void KanagramGame::loadSettings()
 
 void KanagramGame::answerCorrect()
 {
-    m_totalScore += m_correctAnswerScore;
+    if (m_currentPlayerNumber == 1)
+        m_totalScore += m_correctAnswerScore;
+    else
+        m_totalScore2 += m_correctAnswerScore;
     emit scoreChanged();
 }
 
 void KanagramGame::answerIncorrect()
 {
-    m_totalScore += m_incorrectAnswerScore;
+    if (m_currentPlayerNumber == 1)
+        m_totalScore += m_incorrectAnswerScore;
+    else
+        m_totalScore2 += m_incorrectAnswerScore;
     emit scoreChanged();
 }
 
 void KanagramGame::answerRevealed()
 {
-    m_totalScore += m_revealAnswerScore;
+    if (m_currentPlayerNumber == 1)
+        m_totalScore += m_revealAnswerScore;
+    else
+        m_totalScore2 += m_revealAnswerScore;
     emit scoreChanged();
 }
 
 void KanagramGame::answerSkipped()
 {
-    m_totalScore += m_skippedWordScore;
+    if (m_currentPlayerNumber == 1)
+        m_totalScore += m_skippedWordScore;
+    else
+        m_totalScore2 += m_skippedWordScore;
     emit scoreChanged();
 }
